#define MOZ_SOURCE_STAMP aaa0bba23803d2fcf283040f123915cf98a7eef0
#define MOZ_SOURCE_REPO https://hg.mozilla.org/releases/mozilla-esr52
#define MOZ_SOURCE_URL https://hg.mozilla.org/releases/mozilla-esr52/rev/aaa0bba23803d2fcf283040f123915cf98a7eef0
